

####################
# public functions #
####################


### given a tumor genome (or list of genomes) and a set of signatures, decompose each individual tumor

decomposeTumorGenomes <- function(genomes, signatures) {
    # input: genomes = either a list of genome mutation counts as matrices or vectors)
    #                  or a single genome (matrix or vector)
    #        singatures = a list of signatures (matrices or vectors)
    #        [need to have the same format]

    if (!is.list(signatures)) {
        error("Error: Parameter 'signatures' of function 'decomposeTumorGenomes' must be a list of signature objects!")
    }
    if (!is.data.frame(signatures[[1]]) & !is.matrix(signatures[[1]]) & !is.vector(signatures[[1]])) {
        error("Error: signatures for function decomposeTumorGenomes must be data.frames, matrices or vectors!")
    }


    if (!is.list(genomes)) { # it's a single genome, make it a list of one genome
        genomes = list(genomes)
        #name(genomes) = "tumor_genome"
    }

    if (is.null(names(genomes))) { # numbering of genomes if they are not named! (we need the names for the results)
        names(genomes) = paste("genome_", as.character(1:length(genomes)), sep="")
    }

    decompositions  = list()
    
    for (g in 1:length(genomes)) {
        # evaluate each genome individually

        if (!is.data.frame(genomes[[g]]) & !is.matrix(genomes[[g]]) & !is.vector(genomes[[g]])) {
            error("Error: genomes for function decomposeTumorGenomes must be data.frames, matrices or vectors!")
        }
        
        counts = genomes[[g]]

        # first, check its format
        if (length(counts) != length(as.vector(as.matrix(signatures[[1]])))) {
            error("Error: formats of genomes and singatures must match for function decomposeTumorGenomes!")
        }

        decompositions[[g]] = QPforSig(counts, signatures)
        names(decompositions)[g] = names(genomes)[g]
    }

    return(decompositions)
}



### convert tumor genomes that have been read with pmsignature

getGenomesFromMutationFeatureData <- function(countData, normalize=TRUE) {
    # convert MutationFeatureData (e.g. loaded with pmsignature's function readMPFile) to a list of genome matrices

    # IMPORTANT: set normalize=FALSE only if you want to see full counts, but USE ONLY NORMALIZED count tables for
    # predicting/computing exposures, i.e., the contributions of mutational signatures (because these are normalized, too)

    
    if (class(countData) != "MutationFeatureData") {
        error("Error: function getGenomesFromMutationFeatureData requires an object of type MutationFeatureData (as produced by pmsignature's readMPFile)")
    }
    
    # if the user set trDir = TRUE, we need flankinBasesNum+1 rows, otherwise only the flankingBasesNum
    if ((getElement(countData, "transcriptionDirection"))==TRUE) {
        rows <- as.numeric(getElement(countData, "flankingBasesNum")+1)
    } else {
        rows <- as.numeric(getElement(countData, "flankingBasesNum")) 
    }

    tableList <- list()
    #countvecList <- list()
    #countvec <- vector()

    samples <- unique(getElement(countData,"countData")[2,])   # get list of samples (numbers)

    for (l in 1:length(samples)){  # for each sample l

	countTable <- matrix(rep(0, 6*rows), nrow=rows, ncol=6) # initialize the new count table/matrix with 0

        # in oneSample we extract the count data relative to sample l
	oneSample <- getElement(countData, "countData")[,which(getElement(countData, "countData")[2,]==l)]

        
        # (the following work, but might be replaced by apply calls for efficiency)
        
	for (i in 1:ncol(oneSample)) { # for each i in the oneSample vector

            mutTypeIndex <- oneSample[1,i] # index of the mutation type in featureVectorList

            # get the features for this mutation type
            # (corresponds to the column indices to be incremented for each row in the count table)
            v <- getElement(countData,"featureVectorList")[,mutTypeIndex] 

            
            for (k in 1:rows){ #for each k in the length of our rows
                # in our countTable at position k and v[k] we store the number of occurrences 
                # which is stored in the third row and i-th column of oneSample
                countTable[k,v[k]]= countTable[k,v[k]] + oneSample[3,i] 
            }
	}

        # normalize the count table?
        if (normalize) {
            countTable <- countTable/(sum(countTable)/nrow(countTable))
        }
        
        tableList[[l]] <- countTable  # put each countTable in the tableList

    }

    # keep the samples' names/IDs!
    names(tableList) <- getElement(countData, "sampleList")

    return(tableList)
}


### convert a set of signatures from the result of pmsignature

getSignatureListFromEstimatedParameters <- function(Param) {
    # convert the signatures from an "EstimatedParameters" (Param),
    # as obtained from pmsignature, to a list of singature matrices or vectors

    if (class(Param) != "EstimatedParameters") {
        error("Error: function getSignatureListFromEstimatedParameters requires an object of type EstimatedParameters (as produced by pmsignature's getPMSignature)")
    }

    sigList = list()

    # number of signatures
    numSigs = getElement(Param, "signatureNum")
    
    # if one of them was background, the true number of signatures is one less!
    numSigs = numSigs - as.numeric(getElement(Param, "isBackGround"))

    for (sig in 1:numSigs) {
        sigList[[length(sigList)+1]] <- getSignatureValue(Param, sig)
    }

    return(sigList)
}
